/**
*
* Project:     Water Notify for Kadu
* File:        water_notify.cpp
* Copyright:   (C) 2008 by Korneliusz Jrzebski
* Modified by: Tomasz Rostanski <rozteck at interia dot pl>
* Author:      Korneliusz Jarzebski <korneliusz at jarzebski dot pl>
* Link:        http://www.jarzebski.pl/
* Version:     0.3
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*
*/

#include <dbus/dbus.h>
#include <QtGui/QMessageBox>

#include "chat/message/pending-messages-manager.h"
#include "core/core.h"
#include "debug.h"
#include "configuration/configuration-file.h"
#include "gui/widgets/configuration/configuration-widget.h"
#include "misc/path-conversion.h"
#include "notify/notification-manager.h"
#include "notify/notification.h"

#include <X11/Xlib.h>

#include "water_notify.h"


WaterNotify* water_notify = NULL;

extern "C" int water_notify_init()
{
	kdebugf();
	water_notify = new WaterNotify();
	kdebugf2();
	return 0;
}

extern "C" void water_notify_close()
{
	kdebugf();
	delete water_notify;
	water_notify = NULL;
	kdebugf2();
}

NotifierConfigurationWidget* WaterNotify::createConfigurationWidget(QWidget*)
{
	return 0;
}

void WaterNotify::mainConfigurationWindowCreated(MainConfigurationWindow *mainConfigurationWindow)
{
	connect(mainConfigurationWindow->widget()->widgetById("WaterNotify/TestWaterDrop"),
		SIGNAL(clicked()), this, SLOT(ModuleSelfTest()));
}

WaterNotify::WaterNotify(QObject *parent, const char *name) :
		Notifier("Water Notify", "Water Notify", "kadu_icons/notify-hints", parent),
		WaterDrop_On(false), WaterDrop_Disable(false)
{
	kdebugf();
	config_file.addVariable("Water Notify", "RaindropDelay", 1);
	config_file.addVariable("Water Notify", "RaindropAmplitude", 10);

	MainConfigurationWindow::registerUiFile(dataPath("kadu/modules/configuration/water_notify.ui"));
	NotificationManager::instance()->registerNotifier(this);
	timer = new QTimer(this);

	connect(timer, SIGNAL(timeout()), this, SLOT(WaterControl()));
	connect(this, SIGNAL(searchingForTrayPosition(QPoint&)),
		Core::instance(), SIGNAL(searchingForTrayPosition(QPoint&)));
	connect(PendingMessagesManager::instance(), SIGNAL(messageRemoved(Message)),
		this, SLOT(pendingMessageRemoved(Message)));

	DetermineRootWindow();
	kdebugf2();
}

WaterNotify::~WaterNotify()
{
	kdebugf();
	disconnect(timer, SIGNAL(timeout()), this, SLOT(WaterControl()));
	disconnect(this, SIGNAL(searchingForTrayPosition(QPoint&)),
		Core::instance(), SIGNAL(searchingForTrayPosition(QPoint&)));
	disconnect(PendingMessagesManager::instance(), SIGNAL(messageRemoved(Message)),
		this, SLOT(pendingMessageRemoved(Message)));

	NotificationManager::instance()->unregisterNotifier(this);
	MainConfigurationWindow::unregisterUiFile(dataPath("kadu/modules/configuration/water_notify.ui"));
	kdebugf2();
}

void WaterNotify::WaterControl(void)
{
	kdebugf();
	if (!WaterDrop_On)
		return;

	if (WaterDrop_Disable)
	{
		WaterDrop_On = false;
		WaterDrop_Owner = "";
	}
	else
		WaterDrop(false);

	if (!WaterDrop_On)
		timer->stop();
	kdebugf2();
}

void WaterNotify::pendingMessageRemoved(Message message)
{
	kdebugf();

	if (/*WaterDrop_Owner == "NewChat" && */!PendingMessagesManager::instance()->hasPendingMessages())
		WaterDrop_Disable = true;
	else if (WaterDrop_Owner == "NewMessage")
	{
		if (PendingMessagesManager::instance()->pendingMessagesForChat(message.messageChat()).count() ==
			PendingMessagesManager::instance()->pendingMessages().count())
		WaterDrop_Disable = true;
	}
	kdebugf2();
}

void WaterNotify::notify(Notification *notification)
{
	if (WaterDrop_On || ((notification->type() != "NewChat") &&
		(notification->type() != "NewMessage")))
		return;

	if (notification->type() == "NewChat" &&
		!config_file.readBoolEntry("Chat", "OpenChatOnMessage"))
		WaterDrop_On = true;
	else
		WaterDrop_On = false;

	if (WaterDrop_On)
	{
		WaterDrop_Disable = false;
		WaterDrop_Owner = notification->type();
		WaterControl();
		int delay = config_file.readNumEntry("Water Notify", "RaindropDelay");
		timer->start(delay * 1000);
	}
	return;
}

void WaterNotify::AppendArgument_STRING(DBusMessageIter *iter, const char *value)
{
	dbus_message_iter_append_basic (iter, DBUS_TYPE_STRING, &value);
}

void WaterNotify::AppendArgument_INT32(DBusMessageIter *iter, int value)
{
	dbus_message_iter_append_basic (iter, DBUS_TYPE_INT32, &value);
}

void WaterNotify::AppendArgument_DOUBLE(DBusMessageIter *iter, double value)
{
	dbus_message_iter_append_basic (iter, DBUS_TYPE_DOUBLE, &value);
}

void WaterNotify::WaterDrop(bool isTest)
{
	if (RootWindow_ID == 0)
	{
		if (isTest)
			QMessageBox::information(NULL, "Window Notify", tr("Root window failed"));
		return;
	}

	DBusConnection *connection;
	DBusError error;
	DBusMessage *message;
	DBusMessageIter iter;
	DBusBusType type = DBUS_BUS_SESSION;

	dbus_error_init (&error);
	connection = dbus_bus_get (type, &error);
	if (connection == NULL)
	{
		if (isTest)
			QMessageBox::information(NULL, "Window Notify", tr("Failed to open connection : %s").arg(error.message));
		dbus_error_free(&error);
		return;
	}

	message = dbus_message_new_method_call (NULL, "/org/freedesktop/compiz/water/allscreens/point",
		"org.freedesktop.compiz", "activate");

	dbus_message_set_auto_start (message, TRUE);

	if (message == NULL)
	{
		if (isTest)
			QMessageBox::information(NULL, "Window Notify", tr("Couldn't allocate D-Bus message"));
		return;
	}

	if (!dbus_message_set_destination (message, "org.freedesktop.compiz"))
	{
		if (isTest)
			QMessageBox::information(NULL, "Window Notify", tr("Couldn't set message destination\n"));
		return;
	}

	QPoint trayPosition;
	emit searchingForTrayPosition(trayPosition);
	double amplitude = config_file.readNumEntry("Water Notify", "RaindropAmplitude") * 0.01;

	dbus_message_iter_init_append (message, &iter);

	AppendArgument_STRING(&iter, "root");
	AppendArgument_INT32(&iter, RootWindow_ID);
	AppendArgument_STRING(&iter, "amplitude");
	AppendArgument_DOUBLE(&iter, amplitude);
	AppendArgument_STRING(&iter, "x");
	AppendArgument_INT32(&iter, (trayPosition.x() + 10));
	AppendArgument_STRING(&iter, "y");
	AppendArgument_INT32(&iter, (trayPosition.y() + 13));

	dbus_connection_send (connection, message, NULL);
	dbus_connection_flush (connection);
	dbus_message_unref (message);
	dbus_connection_unref (connection);
}

void WaterNotify::DetermineRootWindow(void)
{
	Display *dpy = NULL;
	Window RootWindow = 0;

	dpy = XOpenDisplay(NULL);
	if (dpy == NULL)
	{
		RootWindow_ID = 0;
		return;
	}

	RootWindow = RootWindow(dpy, 0);
	RootWindow_ID = RootWindow;
	XCloseDisplay(dpy);
	dpy = NULL;
	return;
}

void WaterNotify::ModuleSelfTest()
{
	WaterDrop(true);
}
